#!/usr/local/bin/perl -w
# Author: Amine Abou-Rjeili
# Date: 11/12/2002
# Filename: sample2.pl
# Team: Alianza Lima
# Team Members:
# 	Abou-Rjeili, Amine
#	Goopy, Suchitra	
#	Purandare, Amruta	
#	Storie, Sam
# Description: This is an interactive module for the LDOCE interface

#use lib "/home/cs/abou0013/CS8761/project/";
use LDOCE::Interface;

print "*** Welcome to the LDOCE interface test module ***\n";
print "Load LDOCE database at start-up [y/n]? ";
$input = <STDIN>;chomp $input;

$ldoceInit = 0;
$verbose   = 0;

if( $input eq 'y' || $input eq 'Y' ){
    $ldoceInit = 1;
}

print "Enable verbose mode [y/n]? ";
$input = <STDIN>;chomp $input;
if( $input eq 'y' || $input eq 'Y' ){
    $verbose = 1;
}


$ldoce = LDOCE::Interface->new( '', $ldoceInit, $verbose );

# Now let us test some of the functions in the module
$subjectCode = '1';
while( 1 ){
    &menuAndPrompt;
    $input = <STDIN>; chomp $input;

    if( $input eq '1' ){
	&wordRequest;
    }
    elsif( $input eq '2' ){
	&posRequest;
    }
    elsif( $input eq '3' ){
	&subjectCodeRequest;
    }
    elsif( $input eq '4' ){
	&subjectCodeWordRequest;
    }
    elsif( $input eq '5' ){
	&allSubjectCodesRequest;
    }
    elsif( $input eq '6' ){
	&allPosRequest;
    }
    elsif( $input eq '7' ){
	&relatedWordsRequest;
    }
    elsif( $input eq '8' ){
	&multipleWordRequest;
    }
    elsif( $input eq '9' ){
	&subjectCodeForWordRequest;
    }
    elsif( $input eq '10' ){
	&activeCodeForWordRequest;
    }
    elsif( $input eq '11' ){
	&harvestSensesRequest;
    }
    elsif( $input eq '12' ){
	&harvestSensesByWordsRequest;
    }
    elsif( $input eq '0' ){
	last; # Exit
    }
}

sub allSubjectCodesRequest
{
    my( @allCodes ) = ();

    my( $filename );
    my( $input ) = 'y';
    
    *OUTDEST = STDOUT;
    
    print "Redirect output to file? [If yes enter filename else leave blank]: ";
    $filename = <STDIN>; chomp $filename;
    $filename =~ s/^\s+$//; # We do not want a filename with just spaces in it

    if( &chkFile( $filename ) ne '' ){ # Redirect output to a file as requested
	open( OUTFILE, ">$filename" ) ||
	    warn "Could not redirect output to $filename [$!]\n";
	*OUTDEST = OUTFILE;
    }
    else{
	print "(II) Output redirected to terminal\n";
    }

    @allCodes = $ldoce->GetSubjectCodes();
    
    print OUTDEST "Valid subject codes:\n";
    foreach $code (@allCodes){
	print OUTDEST "\t$code\n";
    }
}

sub allPosRequest
{
    my( @allPos ) = ();

    my( $filename );
    my( $input ) = 'y';
    
    *OUTDEST = STDOUT;
    
    print "Redirect output to file? [If yes enter filename else leave blank]: ";
    $filename = <STDIN>; chomp $filename;
    $filename =~ s/^\s+$//; # We do not want a filename with just spaces in it

    if( &chkFile( $filename ) ne '' ){ # Redirect output to a file as requested
	open( OUTFILE, ">$filename" ) ||
	    warn "Could not redirect output to $filename [$!]\n";
	*OUTDEST = OUTFILE;
    }
    else{
	print "(II) Output redirected to terminal\n";
    }

    @allPos = $ldoce->GetPosCodes();
    
    print OUTDEST "Valid Part-Of-Speech codes:\n";
    foreach $code (@allPos){
	print OUTDEST "\t$code\n";
    }
}

sub subjectCodeRequest
{
    my( $pos );
    my( $filename );
    my( $input ) = 'y';
    
    *OUTDEST = STDOUT;
    
    print "Redirect output to file? [If yes enter filename else leave blank]: ";
    $filename = <STDIN>; chomp $filename;
    $filename =~ s/^\s+$//; # We do not want a filename with just spaces in it

    if( &chkFile( $filename ) ne '' ){ # Redirect output to a file as requested
	open( OUTFILE, ">$filename" ) ||
	    warn "Could not redirect output to $filename [$!]\n";
	*OUTDEST = OUTFILE;
    }
    else{
	print "(II) Output redirected to terminal\n";
    }

    print "Enter the subject code: ";
    $subjectCode = <STDIN>;chomp $subjectCode;
        
    @words = $ldoce->QuerySubjectCode($subjectCode, '0');
    
    if( @words == 0 ){
	print "Invalid subject code\n";
    }
    else{
	print OUTDEST "All words that belong to subject code $subjectCode are: \n";
	foreach $word (@words){
	    print OUTDEST "$word\n";
	}
	print OUTDEST "\n";
    }
}

sub subjectCodeWordRequest
{
    my( $subjectCode, $word );
    my( $filename );
    my( $input ) = 'y';
    my( @senses ) = ();

    *OUTDEST = STDOUT;
    
    print "Redirect output to file? [If yes enter filename else leave blank]: ";
    $filename = <STDIN>; chomp $filename;
    $filename =~ s/^\s+$//; # We do not want a filename with just spaces in it

    if( &chkFile( $filename ) ne '' ){ # Redirect output to a file as requested
	open( OUTFILE, ">$filename" ) ||
	    warn "Could not redirect output to $filename [$!]\n";
	*OUTDEST = OUTFILE;
    }
    else{
	print "(II) Output redirected to terminal\n";
    }

    print "Enter the subject code: ";
    $subjectCode = <STDIN>; chomp $subjectCode;
    print "Enter the word: ";
    $word = <STDIN>; chomp $word;
    @senses = $ldoce->QuerySubjectCodeByWord( $word, $subjectCode, '0' );

    print OUTDEST "Senses for word $word in subject code $subjectCode are:\n";
    if( @senses > 0 ){
	foreach $sense (@senses){
	    print OUTDEST "\t* $sense\n";
	}
    }
    else{
	print OUTDEST "\t* None\n";
    }
}

sub posRequest
{
    my( $pos );
    my( $filename );
    my( $input ) = 'y';

    *OUTDEST = STDOUT;

    print "Enter POS: ";
    $pos = <STDIN>; chomp $pos;
    
    print "Redirect output to file? [If yes enter filename else leave blank]: ";
    $filename = <STDIN>; chomp $filename;
    $filename =~ s/^\s+$//; # We do not want a filename with just spaces in it


    if( &chkFile( $filename ) ne '' ){ # Redirect output to a file as requested
	open( OUTFILE, ">$filename" ) ||
	    warn "Could not redirect output to $filename [$!]\n";
	*OUTDEST = OUTFILE;
    }
    else{
	print "(II) Output redirected to terminal\n";
    }

    print OUTDEST "All words with a POS of $pos are:\n";
    @allWords = $ldoce->QueryByPos( $pos );
    foreach $word (@allWords){
	print OUTDEST "$word\n";
    }
    print OUTDEST "** END POS = $pos **\n";   
}

sub wordRequest
{
    my( $word );
    my( $filename );
    my( $input ) = 'y';

    *OUTDEST = STDOUT;
        
    print "Redirect output to file? [If yes enter filename else leave blank]: ";
    $filename = <STDIN>; chomp $filename;
    $filename =~ s/^\s+$//; # We do not want a filename with just spaces in it
    
    if( &chkFile( $filename ) ne '' ){ # Redirect output to a file as requested
	open( OUTFILE, ">$filename" ) ||
	    warn "Could not redirect output to $filename [$!]\n";
	*OUTDEST = OUTFILE;
    }
    else{
	print "(II) Output redirected to terminal\n";
    }

    print "Enter word: ";
    $word = <STDIN>; chomp $word;
                                                                                
    $data = $ldoce->QuerySGML($word);
    @head = $ldoce->GetHead($word);
    @senses = $ldoce->GetSenses($word);
    
    if( !defined( $data ) ){                                                                       
	print "No entry for $word.\n\n";                                
    }                                                                       
    else
    {                                                                       
	#print "Raw sgml:\n$data\n";                                    
	printf OUTDEST "Totals: %2d Head, %2d Senses\n\n", scalar(@head), scalar(@senses);
	for $href ( @head )                                             
	{                                                               
	    print OUTDEST "Head:\n";                                        

	    for $item (keys %$href)
	    {                                                       
		print OUTDEST "\t$item -> $href->{$item}\n";            
	    }                                                       
	    
	    print OUTDEST "\n\n";
	}                                                               
	
	for $href ( @senses )                                           
	{                                                               
	    print OUTDEST "Sense:\n";                                       
	    for $item (keys %$href) 
	    {                                                       
		print OUTDEST "\t$item -> $href->{$item}\n";            
	    }                                                       
	    
	    print OUTDEST "\n\n";                                           
	}                                                               
    }                        
}

sub multipleWordRequest
{
    my( @words );
    my( $filename );
    my( $input ) = 'y';

    *OUTDEST = STDOUT;
    
    print "Redirect output to file? [If yes enter filename else leave blank]: ";
    $filename = <STDIN>; chomp $filename;
    $filename =~ s/^\s+$//; # We do not want a filename with just spaces in it
    
    if( &chkFile( $filename ) ne '' ){ # Redirect output to a file as requested
	open( OUTFILE, ">$filename" ) ||
	    warn "Could not redirect output to $filename [$!]\n";
	*OUTDEST = OUTFILE;
    }
    else{
	print "(II) Output redirected to terminal\n";
    }
    
    print "Enter list of words separated by comma [,]: ";
    $_ = <STDIN>; chomp;
    @words = split( /\, */ );
    
    my( %defns ) = $ldoce->GetSensesMultiple( @words );
    
    my( @senses );
    my( $counter );
    foreach $word (keys %defns){
	$counter = 1;
	print OUTDEST "--- START WORD SENSE: $word\n";
	@senses = @{$defns{$word}};

	print OUTDEST "Word '${word}' not in dictionary\n" if @senses == 0;
	
	foreach $sense (@senses){
	    print OUTDEST "** SENSE $counter:\n";
	    while( ($key, $val) = each %$sense ){
		print OUTDEST "\t$key => $val\n";
	    }
	    $counter ++;
	}
	print OUTDEST "--- END WORD SENSE: $word\n\n";
    }
}

sub relatedWordsRequest
{
    my( $word );
    my( $filename );
    my( $input ) = 'y';
    
    *OUTDEST = STDOUT;
    
    print "Redirect output to file? [If yes enter filename else leave blank]: ";
    $filename = <STDIN>; chomp $filename;
    $filename =~ s/^\s+$//; # We do not want a filename with just spaces in it
    
    if( &chkFile( $filename ) ne '' ){ # Redirect output to a file as requested
	open( OUTFILE, ">$filename" ) ||
	    warn "Could not redirect output to $filename [$!]\n";
	*OUTDEST = OUTFILE;
    }
    else{
	print "(II) Output redirected to terminal\n";
    }

    print "Enter word: ";
    $word = <STDIN>; chomp $word;
    
    my( %result ) = $ldoce->QueryRelated( $word );
    
    print OUTDEST "Word '${word}' not found in the dictionary\n" if !%result;

    my( @allWords );
    foreach $sc (keys %result){
	print OUTDEST "Subject Code: $sc\n";	
	print OUTDEST "\t".join( "\n\t", @{$result{$sc}} )."\n";
    }
}

sub subjectCodeForWordRequest
{
    my( $word );
    my( $filename );
    my( $input ) = 'y';
    
    *OUTDEST = STDOUT;
    
    print "Redirect output to file? [If yes enter filename else leave blank]: ";
    $filename = <STDIN>; chomp $filename;
    $filename =~ s/^\s+$//; # We do not want a filename with just spaces in it
    
    if( &chkFile( $filename ) ne '' ){ # Redirect output to a file as requested
	open( OUTFILE, ">$filename" ) ||
	    warn "Could not redirect output to $filename [$!]\n";
	*OUTDEST = OUTFILE;
    }
    else{
	print "(II) Output redirected to terminal\n";
    }

    print "Enter word: ";
    $word = <STDIN>; chomp $word;

    my( @allCodes ) = $ldoce->GetSubjectCodesForWord( $word );
    
    print OUTDEST "Word '${word}' has no subject codes\n" if @allCodes == 0;

    print OUTDEST "Word: $word\n";
    foreach $code (@allCodes){
	print OUTDEST "\t$code\n";
    }
    
}

sub activeCodeForWordRequest
{
    my( $word );
    my( $filename );
    my( $input ) = 'y';
    
    *OUTDEST = STDOUT;
    
    print "Redirect output to file? [If yes enter filename else leave blank]: ";
    $filename = <STDIN>; chomp $filename;
    $filename =~ s/^\s+$//; # We do not want a filename with just spaces in it
    
    if( &chkFile( $filename ) ne '' ){ # Redirect output to a file as requested
	open( OUTFILE, ">$filename" ) ||
	    warn "Could not redirect output to $filename [$!]\n";
	*OUTDEST = OUTFILE;
    }
    else{
	print "(II) Output redirected to terminal\n";
    }

    print "Enter word: ";
    $word = <STDIN>; chomp $word;

    my( @allCodes ) = $ldoce->GetActivCodesForWord( $word );
    
    print OUTDEST "Word '${word}' has no active codes\n" if @allCodes == 0;

    print OUTDEST "Word: $word\n";
    foreach $code (@allCodes){
	print OUTDEST "\t$code\n";
    }
    
}

sub harvestSensesRequest
{
    my( $word );
    my( $filename );
    my( $input ) = 'y';
    
    *OUTDEST = STDOUT;
    
    print "Redirect output to file? [If yes enter filename else leave blank]: ";
    $filename = <STDIN>; chomp $filename;
    $filename =~ s/^\s+$//; # We do not want a filename with just spaces in it
    
    if( &chkFile( $filename ) ne '' ){ # Redirect output to a file as requested
	open( OUTFILE, ">$filename" ) ||
	    warn "Could not redirect output to $filename [$!]\n";
	*OUTDEST = OUTFILE;
    }
    else{
	print "(II) Output redirected to terminal\n";
    }

    print "Enter word: ";
    $word = <STDIN>; chomp $word;

    my( @allSenses ) = $ldoce->HarvestSenses( $word );
    
    print OUTDEST "Word '${word}' did not appear in any of the senses\n" if @allSenses == 0;

    print OUTDEST "Word: $word\n";
    foreach $sense (@allSenses){
	print OUTDEST "* $sense\n";
    }    
}

sub harvestSensesByWordsRequest
{
    my( $word );
    my( $filename );
    my( $input ) = 'y';
    
    *OUTDEST = STDOUT;
    
    print "Redirect output to file? [If yes enter filename else leave blank]: ";
    $filename = <STDIN>; chomp $filename;
    $filename =~ s/^\s+$//; # We do not want a filename with just spaces in it
    
    if( &chkFile( $filename ) ne '' ){ # Redirect output to a file as requested
	open( OUTFILE, ">$filename" ) ||
	    warn "Could not redirect output to $filename [$!]\n";
	*OUTDEST = OUTFILE;
    }
    else{
	print "(II) Output redirected to terminal\n";
    }

    print "Enter word: ";
    $word = <STDIN>; chomp $word;

    my( @allWords ) = $ldoce->HarvestSensesByWords( $word );
    
    print OUTDEST "Word '${word}' did not appear in any of the senses\n" if @allWords == 0;

    print OUTDEST "Word: $word\n";
    foreach $w (@allWords){
	print OUTDEST "* $w\n";
    }    
}

sub menuAndPrompt
{
    print <<MENU;
1. Query by word
2. Query by POS
3. Query by subject code
4. Query by subject code and word
5. Get all subject codes
6. Get all POS codes
7. Get all related words
8. Query by list of words
9. Get subject codes for a word
10. Get active codes for a word
11. Harvest senses for word
12. Get words that have senses that contain a word
0. Exit
MENU
print "Enter Option: ";
}

sub chkFile( $ )
{
    my( $filename ) = @_;
    my( $input ) = 'y';

    if( $filename ne '' ){
	if( -f $filename ){
	    print "File $filename exists. Overwrite [y/n]? ";
	    $input = <STDIN>; chomp $input;
	}
	
	$filename = '' if( $input eq 'n' );	
    }
    
    return $filename;
}

exit(0);
